﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using Infragistics.Windows.DataPresenter;
using Infragistics.Windows.Editors;

namespace DhpControls
{
    /// <summary>
    /// Führen Sie die Schritte 1a oder 1b und anschließend Schritt 2 aus, um dieses benutzerdefinierte Steuerelement in einer XAML-Datei zu verwenden.
    ///
    /// Schritt 1a) Verwenden des benutzerdefinierten Steuerelements in einer XAML-Datei, die im aktuellen Projekt vorhanden ist.
    /// Fügen Sie dieses XmlNamespace-Attribut dem Stammelement der Markupdatei 
    /// an der Stelle hinzu, an der es verwendet werden soll:
    ///
    ///     xmlns:MyNamespace="clr-namespace:DhpControls"
    ///
    ///
    /// Schritt 1b) Verwenden des benutzerdefinierten Steuerelements in einer XAML-Datei, die in einem anderen Projekt vorhanden ist.
    /// Fügen Sie dieses XmlNamespace-Attribut dem Stammelement der Markupdatei 
    /// an der Stelle hinzu, an der es verwendet werden soll:
    ///
    ///     xmlns:MyNamespace="clr-namespace:DhpControls;assembly=DhpControls"
    ///
    /// Darüber hinaus müssen Sie von dem Projekt, das die XAML-Datei enthält, einen Projektverweis
    /// zu diesem Projekt hinzufügen und das Projekt neu erstellen, um Kompilierungsfehler zu vermeiden:
    ///
    ///     Klicken Sie im Projektmappen-Explorer mit der rechten Maustaste auf das Zielprojekt und anschließend auf
    ///     "Verweis hinzufügen"->"Projekte"->[Navigieren Sie zu diesem Projekt, und wählen Sie es aus.]
    ///
    ///
    /// Schritt 2)
    /// Fahren Sie fort, und verwenden Sie das Steuerelement in der XAML-Datei.
    ///
    ///     <MyNamespace:DhpButtonColumn/>
    ///
    /// </summary>
    [TemplatePart(Name = "PART_DhpButton", Type = typeof(Button))]
    public class DhpButtonEditor : ValueEditor //CellValuePresenter
    {
        static DhpButtonEditor()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(DhpButtonEditor), 
                new FrameworkPropertyMetadata(typeof(DhpButtonEditor)));

            ClickEvent = EventManager.RegisterRoutedEvent("Click",
                                                          RoutingStrategy.Bubble,
                                                          typeof(RoutedEventHandler),
                                                          typeof(DhpMultiColumnComboEditor));
        }

        #region Events

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();

            Button dhpButton = base.GetTemplateChild("PART_DhpButton") as Button;
            if (dhpButton != null)
            {
                dhpButton.Click += new RoutedEventHandler(DhpButton_Click);
            }
        }

        public static readonly RoutedEvent ClickEvent;

        public event RoutedEventHandler Click
        {
            add { AddHandler(ClickEvent, value); }
            remove { RemoveHandler(ClickEvent, value); }
        }

        /// <summary>
        /// http://msdn.microsoft.com/en-us/library/ms752288.aspx#Y228
        /// http://keyvan.io/how-to-add-events-to-custom-wpf-control
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void DhpButton_Click(object sender, RoutedEventArgs e)
        {
            RoutedEventArgs args = new RoutedEventArgs();
            args.RoutedEvent = ClickEvent;
            this.RaiseEvent(args);
        }

        #endregion

        public override bool CanEditType(Type type)
        {
            return true;
        }

        public override bool CanRenderType(Type type)
        {
            return true;
        }
    }
}
