﻿using System;
using System.Collections.ObjectModel;
using System.Data;
using System.Windows;

using Infragistics.Windows.DataPresenter;

namespace DhpControls
{
    /// <summary>
    /// Führen Sie die Schritte 1a oder 1b und anschließend Schritt 2 aus, um dieses benutzerdefinierte Steuerelement in einer XAML-Datei zu verwenden.
    ///
    /// Schritt 1a) Verwenden des benutzerdefinierten Steuerelements in einer XAML-Datei, die im aktuellen Projekt vorhanden ist.
    /// Fügen Sie dieses XmlNamespace-Attribut dem Stammelement der Markupdatei 
    /// an der Stelle hinzu, an der es verwendet werden soll:
    ///
    ///     xmlns:MyNamespace="clr-namespace:DhpControls"
    ///
    ///
    /// Schritt 1b) Verwenden des benutzerdefinierten Steuerelements in einer XAML-Datei, die in einem anderen Projekt vorhanden ist.
    /// Fügen Sie dieses XmlNamespace-Attribut dem Stammelement der Markupdatei 
    /// an der Stelle hinzu, an der es verwendet werden soll:
    ///
    ///     xmlns:MyNamespace="clr-namespace:DhpControls;assembly=DhpControls"
    ///
    /// Darüber hinaus müssen Sie von dem Projekt, das die XAML-Datei enthält, einen Projektverweis
    /// zu diesem Projekt hinzufügen und das Projekt neu erstellen, um Kompilierungsfehler zu vermeiden:
    ///
    ///     Klicken Sie im Projektmappen-Explorer mit der rechten Maustaste auf das Zielprojekt und anschließend auf
    ///     "Verweis hinzufügen"->"Projekte"->[Navigieren Sie zu diesem Projekt, und wählen Sie es aus.]
    ///
    ///
    /// Schritt 2)
    /// Fahren Sie fort, und verwenden Sie das Steuerelement in der XAML-Datei.
    ///
    ///     <MyNamespace:DhpDataGrid/>
    ///
    /// </summary>
    public class DhpDataGrid : XamDataGrid
    {
        static DhpDataGrid()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(DhpDataGrid), 
                new FrameworkPropertyMetadata(typeof(DhpDataGrid)));
        }

        public DHPDataTable dt = new DHPDataTable();

        public DhpDataGrid()
        {
            DhpComboColTest comboColTest = new DhpComboColTest();

            // Create FieldLayout
            FieldLayoutSettings.AutoGenerateFields = false;

            dt.Columns.Add(new DataColumn("Name", typeof(string)));
            dt.Columns.Add(new DataColumn("ComboBox", typeof(DhpComboItem)));
            dt.Columns.Add(new DataColumn("MultiColumnComboBox", typeof(DhpComboItem)));
            dt.Columns.Add(new DataColumn("Button1", typeof(Object)));
            dt.Columns.Add(new DataColumn("Button2", typeof(Object)));

            FieldLayout fieldLayout = new FieldLayout();

            Field field;

            // Field "Name"
            field = new Field("Name");
            field.Width = new FieldLength(100, FieldLengthUnitType.Auto);

            fieldLayout.Fields.Add(field);



            // Field "ComboBox"
            field = new Field("ComboBox");
            field.Width = new FieldLength(100, FieldLengthUnitType.Auto);

            //// - ItemsProvider
            //ComboBoxItemsProvider provider = new ComboBoxItemsProvider();
            //provider.ItemsSource = comboColTest;

            // - Field Editor --> DhpComboEditor
            field.Settings.EditorType = typeof(DhpComboEditor);
            Style comboStyle = new Style(typeof(DhpComboEditor));
            //comboStyle.Setters.Add(new Setter(DhpComboEditor.ValueTypeProperty, typeof(String)));
            //comboStyle.Setters.Add(new Setter(DhpComboEditor.ItemsProviderProperty, provider));
            comboStyle.Setters.Add(new Setter(DhpComboEditor.ItemsSourceProperty, comboColTest));
            comboStyle.Seal();
            field.Settings.EditorStyle = comboStyle;

            fieldLayout.Fields.Add(field);



            // Field "MultiColumnComboBox"
            field = new Field("MultiColumnComboBox");
            field.Width = new FieldLength(100, FieldLengthUnitType.Auto);

            // - Field Editor --> DhpMultiColumnComboEditor
            field.Settings.EditorType = typeof(DhpMultiColumnComboEditor);
            Style multiColumnComboStyle = new Style(typeof(DhpMultiColumnComboEditor));
            multiColumnComboStyle.Setters.Add(new Setter(DhpMultiColumnComboEditor.ItemsSourceProperty, comboColTest));
            multiColumnComboStyle.Seal();
            field.Settings.EditorStyle = multiColumnComboStyle;

            fieldLayout.Fields.Add(field);

            // Field "Button1"
            field = new Field("Button1");
            field.Width = new FieldLength(50, FieldLengthUnitType.Auto);
            field.Settings.EditorType = typeof(DhpButtonEditor);
            
            Style buttonStyle = new Style(typeof(DhpButtonEditor));
            buttonStyle.Setters.Add(new EventSetter(DhpButtonEditor.ClickEvent, new RoutedEventHandler(ButtonClicked)));
            buttonStyle.Seal();
            field.Settings.EditorStyle = buttonStyle;

            fieldLayout.Fields.Add(field);

            // Field "Button2"
            field = new Field("Button2");
            field.Width = new FieldLength(50, FieldLengthUnitType.Auto);

            field.Settings.EditorType = typeof(DhpButtonEditor);

            buttonStyle = new Style(typeof(DhpButtonEditor));
            buttonStyle.Setters.Add(new Setter(DhpButtonEditor.ValueProperty, "Btn 2")); // <-- Why does this not work?  
            buttonStyle.Setters.Add(new EventSetter(DhpButtonEditor.ClickEvent, new RoutedEventHandler(ButtonClicked)));
            buttonStyle.Seal();
            field.Settings.EditorStyle = buttonStyle;

            fieldLayout.Fields.Add(field);

            FieldLayouts.Add(fieldLayout);

            // Fill DataTable
            DataRow dr;

            dr = dt.NewRow();
            dr["Name"] = "Jochen";
            dr["ComboBox"] = comboColTest[0];
            dr["MultiColumnComboBox"] = comboColTest[1];
            dr["Button1"] = "Test1";
            dt.Rows.Add(dr);

            dr = dt.NewRow();
            dr["Name"] = "Christoph";
            dr["ComboBox"] = comboColTest[2];
            dr["MultiColumnComboBox"] = comboColTest[3];
            dr["Button1"] = "Test2";
            dt.Rows.Add(dr);

            DataSource = dt.DefaultView;
        }

        private void ButtonClicked(object sender, RoutedEventArgs e)
        {
            DhpButtonEditor editor = sender as DhpButtonEditor;

            if (editor.Value != null)
                MessageBox.Show(editor.Value.ToString());
            else
                MessageBox.Show("Value = null");
        }
    }

    public class DHPDataTable : DataTable
    {
        protected override DataRow NewRowFromBuilder(DataRowBuilder builder)
        {
            return new DHPDataRow(builder);
        }

        public DHPDataRow DHPRows(int index)
        {
            return (DHPDataRow)Rows[index];
        }

        public DHPDataColumn DHPColumns(int index)
        {
            return (DHPDataColumn)Columns[index];
        }

        public DHPDataColumn DHPColumns(string name)
        {
            return (DHPDataColumn)Columns[name];
        }
    }

    public class DHPDataColumn : DataColumn
    {

    }

    public class DHPDataRow : DataRow
    {
        DHPDataTable _datatable;

        public DHPDataRow(DataRowBuilder rb)
            : base(rb)
        {
            _datatable = (DHPDataTable)Table;
        }
    }

    public class DhpComboItem
    {
        public DhpComboItem(object value, string labelText)
        {
            Value = value;
            LabelText = labelText;
        }

        public object Value { get; set; }

        public string LabelText { get; set; }

        public string DisplayText
        {
            get
            {
                string displayText = string.Empty;

                if (Value != null)
                    displayText = Value.ToString() + "\t";

                return displayText + LabelText;
            }
        }
    }

    public class DhpComboCollection : ObservableCollection<DhpComboItem>
    {

    }

    public class DhpComboColTest : DhpComboCollection
    {
        public DhpComboColTest()
        {
            this.Add(new DhpComboItem(1, "one"));
            this.Add(new DhpComboItem(2, "two"));
            this.Add(new DhpComboItem("drei", "three"));
            this.Add(new DhpComboItem(4, "four"));
            this.Add(new DhpComboItem("fünf", "five"));
            this.Add(new DhpComboItem(6, "six"));
            this.Add(new DhpComboItem(7, "seven"));
            this.Add(new DhpComboItem("acht", "eight"));
        }
    }
}