﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using Infragistics.Controls.Editors;
using Infragistics.Controls.Editors.Primitives;
using Infragistics.Windows;
using Infragistics.Windows.Editors;
using Infragistics.Windows.DataPresenter;

namespace DhpControls
{
    /// <summary>
    /// Führen Sie die Schritte 1a oder 1b und anschließend Schritt 2 aus, um dieses benutzerdefinierte Steuerelement in einer XAML-Datei zu verwenden.
    ///
    /// Schritt 1a) Verwenden des benutzerdefinierten Steuerelements in einer XAML-Datei, die im aktuellen Projekt vorhanden ist.
    /// Fügen Sie dieses XmlNamespace-Attribut dem Stammelement der Markupdatei 
    /// an der Stelle hinzu, an der es verwendet werden soll:
    ///
    ///     xmlns:MyNamespace="clr-namespace:DhpControls"
    ///
    ///
    /// Schritt 1b) Verwenden des benutzerdefinierten Steuerelements in einer XAML-Datei, die in einem anderen Projekt vorhanden ist.
    /// Fügen Sie dieses XmlNamespace-Attribut dem Stammelement der Markupdatei 
    /// an der Stelle hinzu, an der es verwendet werden soll:
    ///
    ///     xmlns:MyNamespace="clr-namespace:DhpControls;assembly=DhpControls"
    ///
    /// Darüber hinaus müssen Sie von dem Projekt, das die XAML-Datei enthält, einen Projektverweis
    /// zu diesem Projekt hinzufügen und das Projekt neu erstellen, um Kompilierungsfehler zu vermeiden:
    ///
    ///     Klicken Sie im Projektmappen-Explorer mit der rechten Maustaste auf das Zielprojekt und anschließend auf
    ///     "Verweis hinzufügen"->"Projekte"->[Navigieren Sie zu diesem Projekt, und wählen Sie es aus.]
    ///
    ///
    /// Schritt 2)
    /// Fahren Sie fort, und verwenden Sie das Steuerelement in der XAML-Datei.
    ///
    ///     <MyNamespace:DhpMultiColumnComboBox/>
    ///
    /// </summary>
    [TemplatePart(Name = "PART_MultiColumnComboTextBlock", Type = typeof(TextBlock))]
    [TemplatePart(Name = "PART_MultiColumnComboEditor", Type = typeof(XamMultiColumnComboEditor))]
    public class DhpMultiColumnComboEditor : XamTextEditor
    {
       static DhpMultiColumnComboEditor()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(DhpMultiColumnComboEditor), 
                new FrameworkPropertyMetadata(typeof(DhpMultiColumnComboEditor)));
        
            ItemsSourceProperty = DependencyProperty.Register("ItemsSource",
                                                              typeof(IEnumerable),
                                                              typeof(DhpMultiColumnComboEditor),
                                                              new UIPropertyMetadata(null,
                                                                                     new PropertyChangedCallback(OnItemsSourceChanged),
                                                                                     new CoerceValueCallback(OnCoerceItemsSource))
                                                              );

            ItemsSourceChangedEvent = EventManager.RegisterRoutedEvent("ItemsSourceChanged",
                                                                       RoutingStrategy.Bubble,
                                                                       typeof(RoutedEventHandler),
                                                                       typeof(DhpMultiColumnComboEditor)
                                                                       );
        }

        #region ItemsSource Property

        public static readonly DependencyProperty ItemsSourceProperty;

        public IEnumerable ItemsSource
        {
            get { return (IEnumerable)GetValue(ItemsSourceProperty); }
            set { SetValue(ItemsSourceProperty, value); }
        }

        private static void OnItemsSourceChanged(DependencyObject o, DependencyPropertyChangedEventArgs e)
        {
            DhpMultiColumnComboEditor dhpMultiColumnComboEditor = o as DhpMultiColumnComboEditor;
            if (dhpMultiColumnComboEditor != null)
                dhpMultiColumnComboEditor.OnItemsSourceChanged((IEnumerable)e.OldValue, (IEnumerable)e.NewValue);
        }

        protected virtual void OnItemsSourceChanged(IEnumerable oldValue, IEnumerable newValue)
        {
            this.RaiseEvent(new RoutedEventArgs(DhpMultiColumnComboEditor.ItemsSourceChangedEvent, this));
        }

        private static object OnCoerceItemsSource(DependencyObject o, Object value)
        {
            DhpMultiColumnComboEditor dhpMultiColumnComboEditor = o as DhpMultiColumnComboEditor;
            if (dhpMultiColumnComboEditor != null)
                return dhpMultiColumnComboEditor.OnCoerceItemsSource((IEnumerable)value);
            else
                return value;
        }

        protected virtual IEnumerable OnCoerceItemsSource(IEnumerable value)
        {
            return value;
        }

        #endregion

        #region ItemsSourceChanged Event

        public static readonly RoutedEvent ItemsSourceChangedEvent;

        public event RoutedEventHandler ItemsSourceChanged
        {
            add { AddHandler(ItemsSourceChangedEvent, value); }
            remove { RemoveHandler(ItemsSourceChangedEvent, value); }
        }

        #endregion

        #region Events

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            
            TemplateContent editTemplateContent = base.EditTemplate.Template;

            if (IsInEditMode)
            {
                XamMultiColumnComboEditor multiColumnComboEditor = base.GetTemplateChild("PART_MultiColumnComboEditor") as XamMultiColumnComboEditor;
                if (multiColumnComboEditor != null)
                {   
                    multiColumnComboEditor.Loaded += new RoutedEventHandler(XamMultiColumnComboEditor_Loaded);
                    multiColumnComboEditor.DropDownClosed += new EventHandler(XamMultiColumnComboEditor_DropDownClosed);
                }
            }
            else
            {
                TextBlock multiColumnComboTextBlock = base.GetTemplateChild("PART_MultiColumnComboTextBlock") as TextBlock;
                if (multiColumnComboTextBlock != null)
                {
                    //TextBlock Events hier einfügen
                }
            } 
        }
   
        private void XamMultiColumnComboEditor_Loaded(object sender, RoutedEventArgs e)
        {
            (sender as XamMultiColumnComboEditor).IsDropDownOpen = true;
            SpecializedTextBox txt = Utilities.GetDescendantFromType(sender as DependencyObject, typeof(SpecializedTextBox), false) as SpecializedTextBox;
            txt.SelectAll();
            txt.Focus();
        }

        private void XamMultiColumnComboEditor_DropDownClosed(object sender, EventArgs e)
        {
            EndEditMode(true, true);
        }

        #endregion
    }
}